/*
 * Copyright (c) 2021-2023 Datalayer, Inc.
 *
 * MIT License
 */
import { Widget } from '@lumino/widgets';
import { JSONExt } from '@lumino/coreutils';
import { getTimeDiff, getTimeString } from './utils';
const EXECUTE_TIME_CLASS = 'dla-ExecTime';
const ANIMATE_TIME_MS = 1000;
const ANIMATE_CSS = `executeHighlight ${ANIMATE_TIME_MS}ms`;
export class ExecTimeWidget extends Widget {
    _panel;
    _cellSlotMap = {};
    _settings = {
        highlight: true,
        positioning: 'left'
    };
    constructor(panel) {
        super();
        this._panel = panel;
        this.updateConnectedCell = this.updateConnectedCell.bind(this);
        const cells = this._panel.context.model.cells;
        cells.changed.connect(this.updateConnectedCell);
        for (let i = 0; i < cells.length; ++i) {
            this._registerMetadataChanges(cells.get(i));
        }
    }
    updateConnectedCell(cells, changed) {
        changed.oldValues.forEach(this._deregisterMetadataChanges.bind(this));
        changed.newValues.forEach(this._registerMetadataChanges.bind(this));
    }
    _registerMetadataChanges(cellModel) {
        if (!(cellModel.id in this._cellSlotMap)) {
            const fn = () => this._cellMetadataChanged(cellModel);
            this._cellSlotMap[cellModel.id] = fn;
            cellModel.metadataChanged.connect(fn);
        }
        this._cellMetadataChanged(cellModel, true);
    }
    _deregisterMetadataChanges(cellModel) {
        const fn = this._cellSlotMap[cellModel.id];
        if (fn) {
            cellModel.metadataChanged.disconnect(fn);
            const codeCell = this._getCodeCell(cellModel);
            if (codeCell) {
                this._removeExecuteNode(codeCell);
            }
        }
        delete this._cellSlotMap[cellModel.id];
    }
    _cellMetadataChanged(cellModel, disableHighlight = false) {
        const codeCell = this._getCodeCell(cellModel);
        if (codeCell) {
            this._updateCodeCell(codeCell, disableHighlight);
        }
        else {
            if (cellModel.type === 'code') {
                console.error(`Could not find code cell for model: ${cellModel}`);
            }
        }
    }
    _getCodeCell(cellModel) {
        if (cellModel.type === 'code') {
            const cell = this._panel.content.widgets.find((widget) => widget.model === cellModel);
            return cell;
        }
        return null;
    }
    _removeExecuteNode(cell) {
        if (cell.inputArea) {
            const editorWidget = cell.inputArea.editorWidget;
            const executionTimeNode = editorWidget.node.querySelector(`.${EXECUTE_TIME_CLASS}`);
            if (executionTimeNode) {
                executionTimeNode.remove();
            }
        }
    }
    _updateCodeCell(cell, disableHighlight) {
        const executionMetadata = cell.model.getMetadata('execution');
        if (executionMetadata && JSONExt.isObject(executionMetadata)) {
            const editorWidget = cell.inputArea.editorWidget;
            let executionTimeNode = editorWidget.node.querySelector(`.${EXECUTE_TIME_CLASS}`);
            if (!executionTimeNode) {
                executionTimeNode = document.createElement('div');
                editorWidget.node.append(executionTimeNode);
            }
            let positioning;
            switch (this._settings.positioning) {
                case 'left':
                    positioning = 'left';
                    break;
                case 'right':
                    positioning = 'right';
                    break;
                default:
                    console.error(`'${positioning}' is not a valid type for the setting 'positioning'`);
            }
            const positioningClass = `${EXECUTE_TIME_CLASS}-positioning-${this._settings.positioning}`;
            executionTimeNode.className = `${EXECUTE_TIME_CLASS} ${positioningClass}`;
            const queuedTimeStr = executionMetadata['iopub.status.busy'];
            const queuedTime = queuedTimeStr ? new Date(queuedTimeStr) : null;
            const startTimeStr = (executionMetadata['shell.execute_reply.started'] ||
                executionMetadata['iopub.execute_input']);
            const startTime = startTimeStr ? new Date(startTimeStr) : null;
            const endTimeStr = executionMetadata['shell.execute_reply'];
            const endTime = endTimeStr ? new Date(endTimeStr) : null;
            let msg = '';
            if (endTime) {
                msg = `Last executed at ${getTimeString(endTime)} in ${getTimeDiff(endTime, startTime)}`;
            }
            else if (startTime) {
                msg = `Execution started at ${getTimeString(startTime)}`;
            }
            else if (queuedTime) {
                msg = `Execution queued at ${getTimeString(queuedTime)}`;
            }
            if (executionTimeNode.innerText !== msg) {
                executionTimeNode.innerText = msg;
                if (!disableHighlight && this._settings.highlight && endTimeStr) {
                    executionTimeNode.style.setProperty('animation', ANIMATE_CSS);
                    setTimeout(() => executionTimeNode.style.removeProperty('animation'), ANIMATE_TIME_MS);
                }
            }
        }
        else {
            this._removeExecuteNode(cell);
        }
    }
}
//# sourceMappingURL=ExecTimeWidget.js.map