import React from 'react';
import { Kernel as JupyterKernel, ServerConnection, ServiceManager } from '@jupyterlab/services';
import { Lite } from './lite';
import { Kernel } from './kernel';
/**
 * The type for Jupyter props.
 */
export type JupyterPropsType = {
    /**
     * Whether the component is collaborative or not.
     */
    collaborative?: boolean;
    /**
     * Default kernel name
     */
    defaultKernelName?: string;
    /**
     * Code to be executed silently at kernel startup
     *
     * This is ignored if there is no default kernel.
     */
    initCode?: string;
    /**
     * URL to fetch a JupyterLite kernel (i.e. in-browser kernel).
     *
     * If defined, {@link serverUrls} and {@link defaultKernelName}
     * will be ignored and the component will run this in-browser
     * kernel.
     *
     * @example
     *
     * https://cdn.jsdelivr.net/npm/@jupyterlite/pyodide-kernel-extension
     */
    lite?: Lite;
    jupyterServerUrl?: string;
    jupyterServerToken?: string;
    serverless?: boolean;
    /**
     * Jupyter Service Manager.
     */
    serviceManager?: ServiceManager.IManager;
    /**
     * Whether to start the default kernel or not.
     */
    startDefaultKernel?: boolean;
    /**
     * A loader to display while the kernel is being setup.
     */
    skeleton?: JSX.Element;
    /**
     * The Kernel Id to use, as defined in the Kernel API
     */
    useRunningKernelId?: string;
    /**
     * The index (aka position) of the Kernel to use in the list of kernels.
     */
    useRunningKernelIndex?: number;
    terminals?: boolean;
};
/**
 * The type for Jupyter context.
 */
export type JupyterContextType = {
    /**
     * Whether the component is collaborative or not.
     */
    collaborative?: boolean;
    /**
     * Default kernel
     */
    defaultKernel?: Kernel;
    /**
     * Will be true while waiting for the default kernel.
     *
     * If `true`, it does not ensure a default kernel will
     * be created successfully.
     *
     * This is useful to not mount to quickly a Lumino Widget
     * to be unmount right away when the default kernel will
     * be available.
     */
    defaultKernelIsLoading: boolean;
    /**
     * Jupyter Server base URL
     *
     * Useless if running an in-browser kernel.
     */
    jupyterServerUrl: string;
    /**
     * Kernel
     */
    kernel?: Kernel;
    /**
     * Kernel
     */
    kernelIsLoading: boolean;
    /**
     * The Kernel Manager.
     */
    kernelManager?: JupyterKernel.IManager;
    /**
     * If `true`, it will load the Pyodide jupyterlite kernel.
     *
     * You can also set it to dynamically import any jupyterlite
     * kernel package.
     *
     * If defined, {@link serverUrls} and {@link defaultKernelName}
     * will be ignored and the component will run this in-browser
     * kernel.
     *
     * @example
     *
     * `lite: true` => Load dynamically the package @jupyterlite/pyodide-kernel-extension
     *
     * `lite: import('@jupyterlite/javascript-kernel-extension')` => Load dynamically
     */
    lite?: Lite;
    serverless: boolean;
    /**
     * Jupyter service manager.
     */
    serviceManager?: ServiceManager.IManager;
    /**
     * Jupyter Server settings.
     *
     * This is useless if running an in-browser kernel via {@link lite}.
     */
    serverSettings?: ServerConnection.ISettings;
};
/**
 * The Jupyter context properties type.
 */
export type JupyterContextProps = React.PropsWithChildren<JupyterPropsType>;
/**
 * The instance for the Jupyter context.
 */
export declare const JupyterContext: React.Context<JupyterContextType | undefined>;
/**
 * The type for the Jupyter context consumer.
 */
export declare const JupyterContextConsumer: React.Consumer<JupyterContextType | undefined>;
export declare const useJupyter: (props?: JupyterPropsType) => JupyterContextType;
/**
 * Utility method to ensure the Jupyter context
 * is authenticated with the Jupyter server.
 */
export declare const ensureJupyterAuth: (serverSettings: ServerConnection.ISettings) => Promise<boolean>;
export declare const createServerSettings: (jupyterServerUrl: string, jupyterServerToken: string) => ServerConnection.ISettings;
/**
 * The Jupyter context provider.
 */
export declare const JupyterContextProvider: React.FC<JupyterContextProps>;
